@@ -1,8 +1,7 @@
 #!/usr/bin/env python3
 
 import asyncio
-from collections import deque
-from pprint import pprint  # noqa: W0611 F401
+from collections import defaultdict, deque
 from threading import Barrier, Thread
 
 import janus
@@ -12,12 +11,15 @@ from .component import Component
 from .pipe import Manifold, Message
 from .utils import aiter, isdict, islist
 
+_events = defaultdict(asyncio.Event)
+
 
 class Node(Component):
     """Processes messages."""
 
     def configure(self, process=None, scale=None, blocking=False,
-                  inbox=None, ifilters=None, ofilters=None, outbox=None, **settings):
+                  inbox=None, ifilters=None, ofilters=None, outbox=None, conditions=None,
+                  **settings):
         if callable(process):
             self.process = process
         self.scale = int(scale or 1)
@@ -26,8 +28,8 @@ class Node(Component):
         self.ifilters = ifilters or PriorityRegistry()
         self.ofilters = ofilters or PriorityRegistry()
         self.outbox = self._join_pipes(outbox, Outbox)
+        self.conditions = conditions or []
         self.layers = list()
-        self.streams = dict()
         return settings
 
     def __str__(self):
@@ -39,11 +41,35 @@ class Node(Component):
     def _join_pipes(self, pipes, class_):
         if not isinstance(pipes, dict):
             pipes = dict(default=pipes)
-        msgbox = self.workflow.make_component(class_, id=self.id, **pipes)
+        msgbox = self.workflow.make_component(class_, id=f'{self.id}.{class_.__name__.lower()}',
+                                              **pipes)
         for channel, pipe in pipes.items():
             pipe.parent = self  # FIXME: other node overwrites .parent
         return msgbox
 
+    @property
+    def status(self):
+        return self._status
+
+    @status.setter
+    def status(self, event):
+        self._status = event
+        self.debug(f'Emitting event {self.id}:{event}')
+        _events[f'{self.id}:{event}'].set()
+
+    async def waiton(self, event):
+        self.debug(f'Waiting on event {event}')
+        await _events[event].wait()
+        self.debug(f'Received event {event}')
+
+    @property
+    def running(self):
+        return self.status in {'started', 'running'}
+
+    @property
+    def stopped(self):
+        return self.status in {'aborted', 'finished'}
+
     def start(self, *args):
         coroutines = [super().start(*args)]
         self.layers = ([self.inbox] +
@@ -56,16 +82,22 @@ class Node(Component):
                 coroutines.append(layer.start())
         return asyncio.gather(*coroutines)
 
-    async def test(self, messages):
-        self.debug(f'~~~~~~ begin {messages}')
-        async for c, m in messages:
-            self.debug(f'~~~~~~ message from {messages} {c} {m}')
-            yield c, m
-            self.debug(f'~~~~~~ yielded from {messages} {c} {m}')
-        self.debug(f'~~~~~~ end {messages}')
-
     async def run(self):
-        await super().run()
+        if self.conditions:
+            for event in self.conditions:
+                await self.waiton(event)
+        self.status = 'running'
+        try:
+            await self._run()
+        except Exception:
+            self.exception()
+            self.status = 'aborted'
+            raise
+
+        else:
+            self.status = 'finished'
+
+    async def _run(self):
         for layer in self.layers:
             if isinstance(layer, Component) and hasattr(layer, 'ready'):
                 self.debug(f'Waiting on layer {layer} to be ready')
@@ -76,11 +108,9 @@ class Node(Component):
             stack = self.layers[0]
             for layer in self.layers[1:]:
                 stack = layer(stack)
-                # stack = layer(self.test(stack))
             self.debug('Processing messages')
             try:
                 async for result in stack:
-                # async for result in self.test(stack):
                     result
             except Exception:
                 self.exception('Node failure')
@@ -92,7 +122,6 @@ class Node(Component):
         processor = self.get_processor()
         if self.blocking:
             raise NotImplementedError(f'Node.blocking in {self}')
-            # processor = ThreadedProcessor(processor, scale=self.scale)
 
         if self.scale == 1:
             return processor
@@ -124,7 +153,6 @@ class Node(Component):
     process.stub = True
 
     def drop(self, message):  # TODO: implement message accounting and leak detection
-        # self.outbox.drop(message)
         raise NotImplementedError(f'drop() in {self}')
 
     def merged_settings(self, message, key=None, msgmap=None):
@@ -143,6 +171,27 @@ class Node(Component):
 
         return Box(settings, **dict((arg, message[msgarg]) for arg, msgarg in msgmap.items()))
 
+    __MISSING__ = object()
+
+    def named_value(self, key, message=None, default=__MISSING__, settings=None):
+        """Return `message`'s attribute named by settings' `key` element.
+
+        If `default` is provided, the element will be given this value if
+        it doesn't exist yet.
+
+        We use the `__MISSING__` value because `None` is a valid default.
+
+        """
+        if settings is None:
+            settings = self._settings
+        if default is not self.__MISSING__:
+            settings.setdefault(key, default)
+        try:
+            return getattr(message, self._settings[key])
+        except AttributeError as e:
+            if default is self.__MISSING__:
+                raise KeyError(*e.args)
+
 
 class WithRetry:
     pass
@@ -156,30 +205,20 @@ class MultiProcessor:
 
     async def feeder(self, messages):
         """Loop through all messages from the inbox and relay them to the input queue."""
-        # self.node.debug(f'*** collector feeder running')
         async for channel, message in messages:
             self.pending += 1
-            # self.node.debug(f'*** collector feeder got msg [{self.pending}] {message}')
             await self.iqueue.put((channel, message))
-            # self.node.debug(f'*** collector feeder sent msg [{self.pending}] {message}')
-        # self.node.debug(f'*** collector feeder done')
         self.exhausted = True
         if self.pending > 0:
             await self.finished.wait()
         for i in range(self.node.scale):
             await self.iqueue.put(Message.EOT)
-        # self.node.debug(f'*** collector feeder exited')
 
     async def collector(self, id_):
         """Launch an instance of the node processor, and relay its output to the output queue."""
-        # self.node.debug(f'*** collector [{id_}] running')
         async for channel, message in self.processor(aiter(self.iqueue.get, Message.EOT), id_=id_):
-            # self.node.debug(f'*** collector [{id_}] got msg {message}')
             await self.oqueue.put((channel, message))
-            # self.node.debug(f'*** collector [{id_}] sent msg {message}')
-        # self.node.debug(f'*** collector [{id_}] stopped')
         await self.oqueue.put((None, Message.EOT))
-        # self.node.debug(f'*** collector [{id_}] exited')
 
     async def outfeed(self, messages):
         """Launch all processor instances and return an iterator that feeds on the output queue."""
@@ -192,27 +231,19 @@ class MultiProcessor:
         for i in range(self.node.scale):
             coroutines.append(self.node.loop.create_task(self.collector(i)))
         running = self.node.scale
-        # self.node.debug(f'*** started {running} collectors')
         while running or not self.oqueue.empty():
             channel, message = await self.oqueue.get()
-            # self.node.debug(f'*** collector outfeed got msg [{self.pending}] {message}')
             self.oqueue.task_done()
             if message is Message.EOT:
                 running -= 1
-                # self.node.debug(f'*** a collector ended, left={running}')
             else:
                 self.pending -= 1
                 if self.pending == 0 and self.exhausted:
-                    # self.node.debug(f'*** collector finishing')
                     self.finished.set()
                 yield channel, message
 
-            # self.node.debug(f'*** collector outfeed processed msg [{self.pending}] {message}')
-
-        # self.node.debug(f'*** waiting on all collectors')
         for coroutine in coroutines:
             await coroutine
-        # self.node.debug(f'*** all collectors ended')
 
     __call__ = outfeed
 
@@ -235,9 +266,7 @@ class Inbox(Manifold):
                 if active_channels == 1:
                     last_channel = True
                 if message is False:  # all channels are empty
-                    # self.debug(f'***** channels sweep empty')
                     await asyncio.sleep(0.05)
-                    # self.debug(f'***** resuming channels sweep')
 
                 message = False
                 active_channels = 0
@@ -245,7 +274,6 @@ class Inbox(Manifold):
 
             active_channels += 1
             try:
-                # self.debug(f'***** waiting for message on channel {channel}')
                 message = await pipe.receive(wait=last_channel)
             except asyncio.QueueEmpty:
                 continue
@@ -276,60 +304,10 @@ class Inbox(Manifold):
                 else:
                     yield channel, pipe
 
-    async def Xreceiver(self):
-        self.debug('Receiving')
-        self.debug(f'Setting up channels {list(self.channels)}')
-        # __import__('pudb').set_trace()
-        # import sys; sys.exit()
-        feeder = deque()
-        events = Box(received=asyncio.Event())
-        # receivers = dict()
-        for channel, pipe in self._active_channels():
-            self.debug(f'Setting up channel {channel}')
-            events[channel] = asyncio.Event()
-            events[channel].set()
-            # receivers[channel] = asyncio.ensure_future(
-            #     self.channel_receiver(channel, feeder, events))
-            asyncio.ensure_future(self.channel_receiver(channel, feeder, events))
-
-        self.debug('Waiting for feeder')
-        while self.running and (len(feeder) or await events.received.wait()):
-            for channel in self.channels:
-                events[channel].clear()
-            events.received.clear()
-            channel, message = feeder.popleft()
-
-            self.debug(f'Got message from {channel}: {message}')
-            if message == Message.EOT:
-                self.debug('Exhausted channel: %s' % channel)
-                pipe.stop()
-                # self.debug(f'* setting event for channel {channel}')
-                events[channel].set()
-                # receivers[channel].cancel()
-                try:
-                    await pipe.send(Message.EOT)
-                except NotImplementedError:
-                    pass
-                # self.debug('-----------------------------')
-                if not list(self._active_channels()):
-                    self.debug('Stopping: no more active channels')
-                    self.stop()
-                    # TODO: cancel channel receiver coroutines
-                    break
-            else:
-                message.checkin(self)
-                yield channel, message
-
-            for channel in self.channels:
-                events[channel].set()
-
-        self.debug('Finished receiving')
-
     async def channel_receiver(self, channel, feeder, events):
         self.debug(f'Receiver for {channel} is ready')
         pipe = self.channels[channel]
         while pipe.running and self.running and await events[channel].wait():
-            # self.debug(f'* receiving from channel {channel}')
             events[channel].clear()
             try:
                 feeder.append((channel, await pipe.receive()))
@@ -337,22 +315,9 @@ class Inbox(Manifold):
                 self.exception(f'Error while receiving on channel {channel}, {pipe}')
                 raise
 
-            # self.debug(f'* received from channel {channel} (next event: {events[channel].is_set()})')  # noqa: E501
             events.received.set()
         self.debug(f'Receiver for {channel} is exiting')
 
-    def X_active_channels(self):
-        channels = self.channels.copy()
-        while channels:
-            for channel, pipe in channels.copy().items():
-                del channels[channel]
-                if not pipe.running:
-                    self.debug(f'skipping stopped channel {channel}')
-                else:
-                    yield channel, pipe
-        if channels:
-            pass  # TODO: channel clean-ups
-
 
 class RetryableMessages:
     """Interfaces a queue between the inbox and the processor where we can insert back retries."""
@@ -362,7 +327,7 @@ class RetryableMessages:
         self.messages = messages
         self._forwarder = None
 
-    async def __aiter__(self):
+    def __aiter__(self):
         if self._forwarder is None:  # singleton forwarder, for when we're using MultiProcessor
             self._forwarder = self.forwarder()
         return self._forwarder
@@ -370,17 +335,13 @@ class RetryableMessages:
     async def feeder(self):
         """Feed the inbox to the queue."""
         async for channel, self.newmsg in self.messages:
-            # self.node.debug(f'*** collector retry: feeding {self.newmsg.type} msg {self.newmsg.meta.id}')  # noqa: E501
             self.forwarded.clear()
             await self.queue.put((channel, self.newmsg))
             await self.forwarded.wait()
-        # self.node.debug('*** collector retry: feeder done')
         self.exhausted = True
         if self.pending > 0:
             await self.finished.wait()
-        # self.node.debug('*** collector retry: feeder ready to exit')
         await self.queue.put(Message.EOT)
-        # self.node.debug('*** collector retry: feeder exited')
 
     async def forwarder(self):
         """Turn the queue into an iterator that the processor can consume."""
@@ -392,28 +353,23 @@ class RetryableMessages:
         self.pending = 0  # number of messages currently handled by node processor
         feeder = self.node.loop.create_task(self.feeder())
         async for channel, message in aiter(self.queue.get, Message.EOT):
-            # self.node.debug(f'*** collector retry: forwarding {message.type} msg {message.meta.id}')  # noqa: E501
             self.queue.task_done()
             if message is self.newmsg:
                 self.forwarded.set()
             self.pending += 1
             yield channel, message
 
-        # self.node.debug(f'*** collector retry: forwarder exiting')
         self.queue.task_done()
         await feeder
-        # self.node.debug(f'*** collector retry: forwarder exited')
 
     def processing_done(self):
         """Signal that a message is not in the processor anymore."""
         self.pending -= 1
-        # self.node.debug(f'*** collector retry: processing done {self.pending}')
         if self.exhausted:
             self.finished.set()
 
     async def retry(self, channel, message, wait=0.001, id_=None):
         """Insert a message back into the queue for retrying it."""
-        # self.node.debug(f'*** collector retry{"" if id_ is None else " %s" % id_}: retrying {message.type} msg {message.meta.id}')  # noqa: E501
         await asyncio.sleep(wait)
         await self.node.loop.create_task(self.queue.put((channel, message)))
         self.processing_done()
@@ -433,21 +389,16 @@ class Outbox(Manifold):
                 self.debug(f'Sending message to {channel}: {message}')
                 if channel in self.channels:
                     await self.channels[channel].send(message)
-                    # self.debug(f'Sent to    {channel}: {message} [{self.channels[channel].id}]')
                     message.checkout(self)
                     yield channel, message
 
-                    # self.debug(f'Yielded    {channel}: {message}')
                 else:
                     raise KeyError(f'Channel "{channel}" does not exist in {self}')
 
         self.debug('Finished sending')
         for pipe in self.channels.values():
-            try:
-                self.debug(f'EOT to {pipe}')
-                await pipe.send(Message.EOT)
-            except Exception:
-                __import__('pudb').set_trace()
+            self.debug(f'EOT to {pipe}')
+            await pipe.send(Message.EOT)
 
 
 class ThreadedNode(Node):
@@ -483,7 +434,7 @@ class ThreadedProcessor:
             # print(f'thread {i} EOT received back')
 
         while self.threads:  # FIXME: join threads instead, and move or retire del self.threads[]
-            asyncio.sleep(0.1)      # while at it, catch exceptions and clean up queue with task_done  # noqa: E501
+            asyncio.sleep(0.1)  #     while at it, catch exceptions and clean up queue with task_done  # noqa: E501
         # print(f'all threads stopped')
 
     async def thread_feeder(self, messages):
