@@ -29,12 +29,7 @@ class Pipe(Component):
 
     def drop(self, message):
         if isinstance(message, Message):
-            try:
-                # self.debug(f'--- dropping {message}')
-                message.drop(self)
-            except Exception:
-                __import__('pudb').set_trace()
-                raise
+            message.drop(self)
 
     async def receive(self, **kwargs):
         raise NotImplementedError(f'receive() out {self}')
@@ -74,12 +69,9 @@ class Manifold(Pipe):
         return '<%s [%s]>' % (self.__class__.__name__,
                               ', '.join('%s:%s' % (n, p.id)
                                         for n, p in self.channels.items()))
-        # ', '.join('%s:%s' % (n, p.get('id', f'--{p}'))
 
     async def run(self):
         await super().run()
-        self.debug('- starting pipes')
-        # __import__('pudb').set_trace()
         for pipe in self.channels.values():
             if not pipe.running:
                 await pipe.start()
@@ -120,9 +112,6 @@ class DataPipe(Pipe):
             except IndexError:
                 break
 
-    def empty(self):
-        return len(self.messages) == 0
-
 
 class QueuePipe(Pipe):
     def configure(self, queue=None, maxsize=1):
