@@ -6,7 +6,6 @@ import sys
 from importlib import import_module
 from inspect import isclass
 from pathlib import Path
-from pprint import pprint  # noqa: W0611 F401
 from string import Template
 
 import strictyaml
@@ -31,8 +30,6 @@ class Workflow:
         self.source = source
         if source == '-':
             source = sys.stdin.read()
-        elif isdict(source):
-            pass  # TODO
         else:
             try:
                 params = os.environ.copy()
@@ -60,9 +57,6 @@ class Workflow:
         elif isdict(element):
             useref = element.pop('<use>', None)
             if useref:
-                # if useref in seen:
-                #     raise ConfigurationError(f'Circular reference detected: {useref}')
-
                 seen.add(useref)
                 try:
                     useref = type(element)(settings[useref].to_dict())
@@ -88,24 +82,18 @@ class Workflow:
     def configure(self):
         """Locate and configure components."""
         _l.debug('Configuring workflow')
-        # pprint(self.app)
         for wname, workflow in self.app.workflows.items():
-            # print(f'workflow: {wname}')
             for nname, node in workflow.items():
                 node.workflow = wname
                 node.key = f'{wname}.{nname}'
-                # print(f'  node: {node.key}')
                 self._configure_node(node)
                 node.setdefault('outbox', dict(default=None))
                 for msgbox in ('inbox', 'outbox'):
                     self._configure_pipes(node, msgbox)
         for wname, workflow in self.app.workflows.items():
-            # print(f'workflow: {wname}')
             for node in workflow.values():
                 for msgbox in ('inbox', 'outbox'):
                     self._configure_connections(node, msgbox)
-        # pprint(self.app)
-        # import sys; sys.exit()
 
     def _configure_node(self, node):
         try:
@@ -114,10 +102,14 @@ class Workflow:
             errmsg = f'Failed to import node "{node.get("component")}" (needed by {node.key})'
             _l.exception(errmsg)
             raise ConfigurationError(errmsg)
+        if 'conditions' in node:
+            if isinstance(node.conditions, str):
+                node.conditions = [node.conditions]
+            for i, condition in enumerate(node.conditions):
+                if '.' not in condition:
+                    node.conditions[i] = f'{node.workflow}.{condition}'
 
     def _configure_pipes(self, node, msgbox):
-        # _l.debug(f'  * {msgbox}: {node.key}')
-
         # Get default channels from class, if any.
         defaults = getattr(node.component, msgbox.upper(), dict())
         if defaults:
@@ -141,8 +133,6 @@ class Workflow:
             if not spec:
                 spec = 'pipekit.pipe:QueuePipe'
             if isstr(spec):
-                # if spec == '<void>':
-                #     spec = 'pipekit.pipe:NullPipe'
                 if spec.startswith('<'):
                     break  # reference will be later resolved in _configure_connections()
 
@@ -158,13 +148,9 @@ class Workflow:
                 channel.component = resolve(channel.component)
             channels[name] = channel
 
-        # print(' mb 2', channels)
         node[msgbox] = channels
-        # _l.debug(f'  ** {msgbox}: {node.key}')
 
     def _configure_connections(self, node, msgbox):
-        # _l.debug(f'  * {msgbox}: {node.key}')
-
         for name, channel in node[msgbox].items():
             if isstr(channel):
                 if not channel.startswith('<'):
@@ -175,10 +161,6 @@ class Workflow:
                 peer_node, peer_channel = self.peer_node(spec, node)
                 peer_msgbox = 'outbox' if msgbox == 'inbox' else 'inbox'
                 node[msgbox][name] = PipeRef(peer_node, peer_msgbox, peer_channel)
-                _l.debug(f'  ** new PipeRef: {node.key} < '
-                         f'{peer_node.key}.{peer_msgbox}:{peer_channel}')
-
-        # _l.debug(f'  ** {msgbox}: {node.key}')
 
     def build(self):
         """Instantiate and wire up nodes and pipes."""
@@ -194,8 +176,6 @@ class Workflow:
             except Exception:
                 _l.exception(f'Error while instantiating node {node.key}')
                 raise
-        # pprint(self.app)
-        # import sys; sys.exit()
 
     RESERVED_SETTINGS = set(['id', 'process', 'scale', 'inbox', 'ifilters', 'ofilters', 'outbox'])
 
@@ -204,8 +184,6 @@ class Workflow:
         if node.key not in self._node_backlog:
             return
 
-        _l.debug(f'  node: {node.key}')
-        _l.debug(f'  settings: {node.get("settings", dict())}')
         settings = node.get('settings', {})
         reserved_settings = self.RESERVED_SETTINGS.intersection(set(settings.keys()))
         if reserved_settings:
@@ -221,34 +199,27 @@ class Workflow:
 
         inbox = dict()
         for channel, pipe in node.inbox.items():
-            # _l.debug(f'  inbox.{channel}: {pipe}')
             if hasattr(pipe, 'instance'):
-                # _l.debug(f'  * instance exists: {pipe.instance}')
                 inbox[channel] = pipe.instance
                 raise Exception('node instance exists')
 
             if isinstance(pipe, PipeRef):
-                # _l.debug(f'  * connection {pipe}')
                 try:
                     node.inbox[channel] = pipe = Box(component=pipe, instance=pipe.resolve())
                 except KeyError:
                     raise ConfigurationError(f'Pipe not found: {pipe}')
 
-                # _l.debug(f'  * connected {pipe}')
-            else:  # if 'component' in pipe:
-              try:  # noqa: E111
-                # _l.debug(f'  * component type: {type(pipe.component)}')
+            else:
                 pipe.instance = self.make_component(
                     pipe.component, id=f'{node.key}.input.{channel}', **pipe.get('settings', {}))
-              except:  # noqa: E111 E722
-                import pudb; pudb.set_trace()  # noqa: E702
             inbox[channel] = pipe.instance
 
         ifilters = self._make_filters(node, 'ifilters')
         ofilters = self._make_filters(node, 'ofilters')
         node_args = dict(
             id=node.key, blocking=bool(node.get('blocking')), scale=node.get('scale'), inbox=inbox,
-            ifilters=ifilters, ofilters=ofilters, outbox=outbox, **settings)
+            ifilters=ifilters, ofilters=ofilters, outbox=outbox,
+            conditions=node.get('conditions', []), **settings)
         if isclass(node.component) and issubclass(node.component, Node):
             node_class = node.component
         elif callable(node.component):
@@ -260,18 +231,15 @@ class Workflow:
                 f'got {type(node.component)} instead')
 
         node.instance = self.make_component(node_class, **node_args)
-        # print(f'    instance: {node.instance}')
         del self._node_backlog[node.key]
 
     def _make_filters(self, node, type_):
         """Instantiate and return filters wrapped in a PriorityRegistry."""
         filters = node.get(type_, {})
-        # pprint(filters)
         for name, filter_ in filters.items():
             filter_.instance = self.make_component(
                 resolve(filter_.component), id=f'{node.key}.{type_}.{name}',
                 **filter_.get('settings', {}))
-            # pprint([name, filter_])
         return PriorityRegistry(dict((k, f.instance) for k, f in filters.items()))
 
     def peer_node(self, spec, dependent):
@@ -320,13 +288,6 @@ def resolve(spec):
     """Parse component textual spec and import and return corresponding object."""
     try:
         module, attr, *_ = spec.rsplit(':', 1) + [None]
-        # nameparts = module.split('.')
-        # if nameparts[0] == 'pipekit':
-        #     module = '.'.join(nameparts[1:])
-        #     parent = 'pipekit'
-        # else:
-        #     parent = 'components'
-        # module = import_module(f'{parent}.{module}')
         module = import_module(module)
         return getattr(module, attr)
 
